=== modified file 'bzrlib/diff.py'
--- bzrlib/diff.py	2006-05-23 01:07:31 +0000
+++ bzrlib/diff.py	2006-05-29 01:00:30 +0000
@@ -17,11 +17,13 @@
 from bzrlib.delta import compare_trees
 from bzrlib.errors import BzrError
 import bzrlib.errors as errors
-from bzrlib.patiencediff import SequenceMatcher, unified_diff
+from bzrlib.patiencediff import unified_diff
+import bzrlib.patiencediff
 from bzrlib.symbol_versioning import *
 from bzrlib.textfile import check_text_lines
 from bzrlib.trace import mutter
 
+
 # TODO: Rather than building a changeset object, we should probably
 # invoke callbacks on an object.  That object can either accumulate a
 # list, write them out directly, etc etc.
@@ -49,7 +51,7 @@
         check_text_lines(newlines)
 
     if sequence_matcher is None:
-        sequence_matcher = SequenceMatcher
+        sequence_matcher = bzrlib.patiencediff.PatienceSequenceMatcher
     ud = unified_diff(oldlines, newlines,
                       fromfile=old_filename+'\t', 
                       tofile=new_filename+'\t',

=== modified file 'bzrlib/knit.py'
--- bzrlib/knit.py	2006-05-17 07:09:13 +0000
+++ bzrlib/knit.py	2006-05-25 14:25:24 +0000
@@ -120,7 +120,7 @@
         """Generate line-based delta from this content to new_lines."""
         new_texts = [text for origin, text in new_lines._lines]
         old_texts = [text for origin, text in self._lines]
-        s = SequenceMatcher(None, old_texts, new_texts)
+        s = KnitSequenceMatcher(None, old_texts, new_texts)
         for op in s.get_opcodes():
             if op[0] == 'equal':
                 continue
@@ -427,7 +427,7 @@
             else:
                 old_texts = []
             new_texts = new_content.text()
-            delta_seq = SequenceMatcher(None, old_texts, new_texts)
+            delta_seq = KnitSequenceMatcher(None, old_texts, new_texts)
             return parent, sha1, noeol, self._make_line_delta(delta_seq, new_content)
         else:
             delta = self.factory.parse_line_delta(data, version_idx)
@@ -481,7 +481,7 @@
             delta_seq = None
             for parent_id in parents:
                 merge_content = self._get_content(parent_id, parent_texts)
-                seq = SequenceMatcher(None, merge_content.text(), content.text())
+                seq = KnitSequenceMatcher(None, merge_content.text(), content.text())
                 if delta_seq is None:
                     # setup a delta seq to reuse.
                     delta_seq = seq
@@ -498,7 +498,7 @@
                 reference_content = self._get_content(parents[0], parent_texts)
                 new_texts = content.text()
                 old_texts = reference_content.text()
-                delta_seq = SequenceMatcher(None, old_texts, new_texts)
+                delta_seq = KnitSequenceMatcher(None, old_texts, new_texts)
             return self._make_line_delta(delta_seq, content)
 
     def _make_line_delta(self, delta_seq, new_content):
@@ -861,7 +861,7 @@
         annotated_b = self.annotate(ver_b)
         plain_a = [t for (a, t) in annotated_a]
         plain_b = [t for (a, t) in annotated_b]
-        blocks = SequenceMatcher(None, plain_a, plain_b).get_matching_blocks()
+        blocks = KnitSequenceMatcher(None, plain_a, plain_b).get_matching_blocks()
         a_cur = 0
         b_cur = 0
         for ai, bi, l in blocks:
@@ -1603,7 +1603,7 @@
 InterVersionedFile.register_optimiser(WeaveToKnit)
 
 
-class SequenceMatcher(difflib.SequenceMatcher):
+class KnitSequenceMatcher(difflib.SequenceMatcher):
     """Knit tuned sequence matcher.
 
     This is based on profiling of difflib which indicated some improvements

=== modified file 'bzrlib/merge3.py'
--- bzrlib/merge3.py	2006-05-23 02:06:15 +0000
+++ bzrlib/merge3.py	2006-05-29 00:58:36 +0000
@@ -20,9 +20,10 @@
 
 
 from bzrlib.errors import CantReprocessAndShowBase
-from bzrlib.patiencediff import SequenceMatcher
+import bzrlib.patiencediff
 from bzrlib.textfile import check_text_lines
 
+
 def intersect(ra, rb):
     """Given two ranges return the range where they intersect or None.
 
@@ -293,8 +294,8 @@
             type, iz, zmatch, ia, amatch, ib, bmatch = region
             a_region = self.a[ia:amatch]
             b_region = self.b[ib:bmatch]
-            matches = SequenceMatcher(None, a_region, 
-                                      b_region).get_matching_blocks()
+            matches = bzrlib.patiencediff.PatienceSequenceMatcher(
+                    None, a_region, b_region).get_matching_blocks()
             next_a = ia
             next_b = ib
             for region_ia, region_ib, region_len in matches[:-1]:
@@ -326,8 +327,10 @@
         """
 
         ia = ib = 0
-        amatches = SequenceMatcher(None, self.base, self.a).get_matching_blocks()
-        bmatches = SequenceMatcher(None, self.base, self.b).get_matching_blocks()
+        amatches = bzrlib.patiencediff.PatienceSequenceMatcher(
+                None, self.base, self.a).get_matching_blocks()
+        bmatches = bzrlib.patiencediff.PatienceSequenceMatcher(
+                None, self.base, self.b).get_matching_blocks()
         len_a = len(amatches)
         len_b = len(bmatches)
 
@@ -383,8 +386,10 @@
 
     def find_unconflicted(self):
         """Return a list of ranges in base that are not conflicted."""
-        am = SequenceMatcher(None, self.base, self.a).get_matching_blocks()
-        bm = SequenceMatcher(None, self.base, self.b).get_matching_blocks()
+        am = bzrlib.patiencediff.PatienceSequenceMatcher(
+                None, self.base, self.a).get_matching_blocks()
+        bm = bzrlib.patiencediff.PatienceSequenceMatcher(
+                None, self.base, self.b).get_matching_blocks()
 
         unc = []
 

=== modified file 'bzrlib/patiencediff.py'
--- bzrlib/patiencediff.py	2006-05-23 14:02:57 +0000
+++ bzrlib/patiencediff.py	2006-05-29 00:50:34 +0000
@@ -23,8 +23,10 @@
 import sys
 import time
 
-
-__all__ = ['SequenceMatcher', 'unified_diff', 'unified_diff_files']
+from bzrlib.trace import mutter
+
+
+__all__ = ['PatienceSequenceMatcher', 'unified_diff', 'unified_diff_files']
 
 
 def unique_lcs(a, b):
@@ -105,50 +107,55 @@
     return result
 
 
-def recurse_matches(a, b, ahi, bhi, answer, maxrecursion):
+def recurse_matches(a, b, alo, blo, ahi, bhi, answer, maxrecursion):
     """Find all of the matching text in the lines of a and b.
 
     :param a: A sequence
     :param b: Another sequence
+    :param alo: The start location of a to check, typically 0
+    :param ahi: The start location of b to check, typically 0
     :param ahi: The maximum length of a to check, typically len(a)
     :param bhi: The maximum length of b to check, typically len(b)
     :param answer: The return array. Will be filled with tuples
-                   indicating [(line_in_a), (line_in_b)]
+                   indicating [(line_in_a, line_in_b)]
     :param maxrecursion: The maximum depth to recurse.
                          Must be a positive integer.
     :return: None, the return value is in the parameter answer, which
              should be a list
 
     """
-    oldlen = len(answer)
     if maxrecursion < 0:
+        mutter('max recursion depth reached')
         # this will never happen normally, this check is to prevent DOS attacks
         return
     oldlength = len(answer)
-    if len(answer) == 0:
-        alo, blo = 0, 0
-    else:
-        alo, blo = answer[-1]
-        alo += 1
-        blo += 1
     if alo == ahi or blo == bhi:
         return
+    last_a_pos = alo-1
+    last_b_pos = blo-1
     for apos, bpos in unique_lcs(a[alo:ahi], b[blo:bhi]):
         # recurse between lines which are unique in each file and match
         apos += alo
         bpos += blo
-        recurse_matches(a, b, apos, bpos, answer, maxrecursion - 1)
+        # Most of the time, you will have a sequence of similar entries
+        if last_a_pos+1 != apos or last_b_pos+1 != bpos:
+            recurse_matches(a, b, last_a_pos+1, last_b_pos+1,
+                apos, bpos, answer, maxrecursion - 1)
+        last_a_pos = apos
+        last_b_pos = bpos
         answer.append((apos, bpos))
     if len(answer) > oldlength:
         # find matches between the last match and the end
-        recurse_matches(a, b, ahi, bhi, answer, maxrecursion - 1)
+        recurse_matches(a, b, last_a_pos+1, last_b_pos+1,
+                        ahi, bhi, answer, maxrecursion - 1)
     elif a[alo] == b[blo]:
         # find matching lines at the very beginning
         while alo < ahi and blo < bhi and a[alo] == b[blo]:
             answer.append((alo, blo))
             alo += 1
             blo += 1
-        recurse_matches(a, b, ahi, bhi, answer, maxrecursion - 1)
+        recurse_matches(a, b, alo, blo,
+                        ahi, bhi, answer, maxrecursion - 1)
     elif a[ahi - 1] == b[bhi - 1]:
         # find matching lines at the very end
         nahi = ahi - 1
@@ -156,104 +163,95 @@
         while nahi > alo and nbhi > blo and a[nahi - 1] == b[nbhi - 1]:
             nahi -= 1
             nbhi -= 1
-        recurse_matches(a, b, nahi, nbhi, answer, maxrecursion - 1)
+        recurse_matches(a, b, last_a_pos+1, last_b_pos+1,
+                        nahi, nbhi, answer, maxrecursion - 1)
         for i in xrange(ahi - nahi):
             answer.append((nahi + i, nbhi + i))
 
 
-class SequenceMatcher(difflib.SequenceMatcher):
+def _collapse_sequences(matches):
+    """Find sequences of lines.
+
+    Given a sequence of [(line_in_a, line_in_b),]
+    find regions where they both increment at the same time
+    """
+    answer = []
+    start_a = start_b = None
+    length = 0
+    for i_a, i_b in matches:
+        if (start_a is not None
+            and (i_a == start_a + length) 
+            and (i_b == start_b + length)):
+            length += 1
+        else:
+            if start_a is not None:
+                answer.append((start_a, start_b, length))
+            start_a = i_a
+            start_b = i_b
+            length = 1
+
+    if length != 0:
+        answer.append((start_a, start_b, length))
+
+    return answer
+
+
+def _check_consistency(answer):
+    # For consistency sake, make sure all matches are only increasing
+    next_a = -1
+    next_b = -1
+    for a,b,match_len in answer:
+        assert a >= next_a, 'Non increasing matches for a'
+        assert b >= next_b, 'Not increasing matches for b'
+        next_a = a + match_len
+        next_b = b + match_len
+
+
+class PatienceSequenceMatcher(difflib.SequenceMatcher):
     """Compare a pair of sequences using longest common subset."""
 
+    _do_check_consistency = True
+
     def __init__(self, isjunk=None, a='', b=''):
         if isjunk is not None:
             raise NotImplementedError('Currently we do not support'
                                       ' isjunk for sequence matching')
         difflib.SequenceMatcher.__init__(self, isjunk, a, b)
 
-    def _check_with_diff(self, alo, ahi, blo, bhi, answer):
-        """Use the original diff algorithm on an unmatched section.
-
-        This will check to make sure the range is worth checking,
-        before doing any work.
-
-        :param alo: The last line that actually matched
-        :param ahi: The next line that actually matches
-        :param blo: Same as alo, only for the 'b' set
-        :param bhi: Same as ahi
-        :param answer: An array which will have the new ranges appended to it
-        :return: None
+    def get_matching_blocks(self):
+        """Return list of triples describing matching subsequences.
+
+        Each triple is of the form (i, j, n), and means that
+        a[i:i+n] == b[j:j+n].  The triples are monotonically increasing in
+        i and in j.
+
+        The last triple is a dummy, (len(a), len(b), 0), and is the only
+        triple with n==0.
+
+        >>> s = PatienceSequenceMatcher(None, "abxcd", "abcd")
+        >>> s.get_matching_blocks()
+        [(0, 0, 2), (3, 2, 2), (5, 4, 0)]
         """
-        # WORKAROUND
-        # recurse_matches has an implementation design
-        # which does not match non-unique lines in the
-        # if they do not touch matching unique lines
-        # so we rerun the regular diff algorithm
-        # if find a large enough chunk.
-
-        # recurse_matches already looked at the direct
-        # neighbors, so we only need to run if there is
-        # enough space to do so
-        if ahi - alo > 2 and bhi - blo > 2:
-            a = self.a[alo+1:ahi-1]
-            b = self.b[blo+1:bhi-1]
-            m = difflib.SequenceMatcher(None, a, b)
-            new_blocks = m.get_matching_blocks()
-            # difflib always adds a final match
-            new_blocks.pop()
-            for blk in new_blocks:
-                answer.append((blk[0]+alo+1,
-                               blk[1]+blo+1,
-                               blk[2]))
-
-    def __helper(self, alo, ahi, blo, bhi, answer):
+        # jam 20060525 This is the python 2.4.1 difflib get_matching_blocks 
+        # implementation which uses __helper. 2.4.3 got rid of helper for
+        # doing it inline with a queue.
+        # We should consider doing the same for recurse_matches
+
+        if self.matching_blocks is not None:
+            return self.matching_blocks
+
         matches = []
-        a = self.a[alo:ahi]
-        b = self.b[blo:bhi]
-        recurse_matches(a, b, len(a), len(b), matches, 10)
+        recurse_matches(self.a, self.b, 0, 0,
+                        len(self.a), len(self.b), matches, 10)
         # Matches now has individual line pairs of
         # line A matches line B, at the given offsets
-
-        start_a = start_b = None
-        length = 0
-        for i_a, i_b in matches:
-            if (start_a is not None
-                and (i_a == start_a + length) 
-                and (i_b == start_b + length)):
-                length += 1
-            else:
-                # New block
-                if start_a is None:
-                    # We need to check from 0,0 until the current match
-                    self._check_with_diff(alo-1, i_a+alo, blo-1, i_b+blo, 
-                                          answer)
-                else:
-                    answer.append((start_a+alo, start_b+blo, length))
-                    self._check_with_diff(start_a+alo+length, i_a+alo,
-                                          start_b+blo+length, i_b+blo,
-                                          answer)
-
-                start_a = i_a
-                start_b = i_b
-                length = 1
-
-        if length != 0:
-            answer.append((start_a+alo, start_b+blo, length))
-            self._check_with_diff(start_a+alo+length, ahi+1,
-                                  start_b+blo+length, bhi+1,
-                                  answer)
-        if not matches:
-            # Nothing matched, so we need to send the complete text
-            self._check_with_diff(alo-1, ahi+1, blo-1, bhi+1, answer)
-
-        # For consistency sake, make sure all matches are only increasing
-        if __debug__:
-            next_a = -1
-            next_b = -1
-            for a,b,match_len in answer:
-                assert a >= next_a, 'Non increasing matches for a'
-                assert b >= next_b, 'Not increasing matches for b'
-                next_a = a + match_len
-                next_b = b + match_len
+        self.matching_blocks = _collapse_sequences(matches)
+        self.matching_blocks.append( (len(self.a), len(self.b), 0) )
+        if PatienceSequenceMatcher._do_check_consistency:
+            if __debug__:
+                _check_consistency(self.matching_blocks)
+
+        return self.matching_blocks
 
 
 # This is a version of unified_diff which only adds a factory parameter
@@ -354,12 +352,12 @@
     import optparse
     p = optparse.OptionParser(usage='%prog [options] file_a file_b'
                                     '\nFiles can be "-" to read from stdin')
-    p.add_option('--cdv', dest='matcher', action='store_const', const='cdv',
-                 default='cdv', help='Use the cdv difference algorithm')
+    p.add_option('--patience', dest='matcher', action='store_const', const='patience',
+                 default='patience', help='Use the patience difference algorithm')
     p.add_option('--difflib', dest='matcher', action='store_const', const='difflib',
-                 default='cdv', help='Use python\'s difflib algorithm')
+                 default='patience', help='Use python\'s difflib algorithm')
 
-    algorithms = {'cdv':SequenceMatcher, 'difflib':difflib.SequenceMatcher}
+    algorithms = {'patience':PatienceSequenceMatcher, 'difflib':difflib.SequenceMatcher}
 
     (opts, args) = p.parse_args(args)
     matcher = algorithms[opts.matcher]

=== modified file 'bzrlib/tests/test_diff.py'
--- bzrlib/tests/test_diff.py	2006-05-23 14:02:57 +0000
+++ bzrlib/tests/test_diff.py	2006-05-29 00:50:34 +0000
@@ -1,9 +1,24 @@
+# Copyright (C) 2005, 2006 Canonical Development Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
 from cStringIO import StringIO
 
 from bzrlib.diff import internal_diff
 from bzrlib.errors import BinaryFile
-from bzrlib.patiencediff import (recurse_matches, SequenceMatcher, unique_lcs,
-                                 unified_diff, unified_diff_files)
+import bzrlib.patiencediff
 from bzrlib.tests import TestCase, TestCaseInTempDir
 
 
@@ -62,9 +77,10 @@
         udiff_lines([], [1023 * 'a' + '\x00'], allow_binary=True)
 
 
-class TestCDVDiffLib(TestCase):
+class TestPatienceDiffLib(TestCase):
 
     def test_unique_lcs(self):
+        unique_lcs = bzrlib.patiencediff.unique_lcs
         self.assertEquals(unique_lcs('', ''), [])
         self.assertEquals(unique_lcs('a', 'a'), [(0,0)])
         self.assertEquals(unique_lcs('a', 'b'), [])
@@ -78,10 +94,11 @@
     def test_recurse_matches(self):
         def test_one(a, b, matches):
             test_matches = []
-            recurse_matches(a, b, len(a), len(b), test_matches, 10)
+            bzrlib.patiencediff.recurse_matches(a, b, 0, 0, len(a), len(b),
+                test_matches, 10)
             self.assertEquals(test_matches, matches)
 
-        test_one(['a', None, 'b', None, 'c'], ['a', 'a', 'b', 'c', 'c'],
+        test_one(['a', '', 'b', '', 'c'], ['a', 'a', 'b', 'c', 'c'],
                  [(0, 0), (2, 2), (4, 4)])
         test_one(['a', 'c', 'b', 'a', 'c'], ['a', 'b', 'c'],
                  [(0, 0), (2, 1), (4, 2)])
@@ -97,14 +114,13 @@
         test_one('aBccDe', 'abccde', [(0,0), (5,5)])
 
     def test_matching_blocks(self):
-        def chk_blocks(a, b, matching):
+        def chk_blocks(a, b, expected_blocks):
             # difflib always adds a signature of the total
             # length, with no matching entries at the end
-            s = SequenceMatcher(None, a, b)
+            s = bzrlib.patiencediff.PatienceSequenceMatcher(None, a, b)
             blocks = s.get_matching_blocks()
-            x = blocks.pop()
-            self.assertEquals(x, (len(a), len(b), 0))
-            self.assertEquals(matching, blocks)
+            self.assertEquals((len(a), len(b), 0), blocks[-1])
+            self.assertEquals(expected_blocks, blocks[:-1])
 
         # Some basic matching tests
         chk_blocks('', '', [])
@@ -133,19 +149,28 @@
                     'how are you today?\n'],
                 [(0, 0, 1), (2, 1, 1)])
 
-        chk_blocks('aBccDe', 'abccde', [(0,0,1), (2,2,2), (5,5,1)])
-
-        chk_blocks('aBcdEcdFg', 'abcdecdfg', [(0,0,1), (2,2,2),
-                                              (5,5,2), (8,8,1)])
-
-        chk_blocks('abbabbXd', 'cabbabxd', [(0,1,5), (7,7,1)])
-        chk_blocks('abbabbbb', 'cabbabbc', [(0,1,6)])
-        chk_blocks('bbbbbbbb', 'cbbbbbbc', [(0,1,6)])
+        # non unique lines surrounded by non-matching lines
+        # won't be found
+        chk_blocks('aBccDe', 'abccde', [(0,0,1), (5,5,1)])
+
+        # But they only need to be locally unique
+        chk_blocks('aBcDec', 'abcdec', [(0,0,1), (2,2,1), (4,4,2)])
+
+        # non unique blocks won't be matched
+        chk_blocks('aBcdEcdFg', 'abcdecdfg', [(0,0,1), (8,8,1)])
+
+        # but locally unique ones will
+        chk_blocks('aBcdEeXcdFg', 'abcdecdfg', [(0,0,1), (2,2,2),
+                                              (5,4,1), (7,5,2), (10,8,1)])
+
+        chk_blocks('abbabbXd', 'cabbabxd', [(7,7,1)])
+        chk_blocks('abbabbbb', 'cabbabbc', [])
+        chk_blocks('bbbbbbbb', 'cbbbbbbc', [])
 
     def test_opcodes(self):
-        def chk_ops(a, b, codes):
-            s = SequenceMatcher(None, a, b)
-            self.assertEquals(codes, s.get_opcodes())
+        def chk_ops(a, b, expected_codes):
+            s = bzrlib.patiencediff.PatienceSequenceMatcher(None, a, b)
+            self.assertEquals(expected_codes, s.get_opcodes())
 
         chk_ops('', '', [])
         chk_ops([], [], [])
@@ -185,36 +210,48 @@
                 , 'how are you today?\n'],
                 [('equal',  0,1, 0,1),
                  ('delete', 1,2, 1,1),
-                 ('equal',  2,3, 1,2)
+                 ('equal',  2,3, 1,2),
                 ])
         chk_ops('aBccDe', 'abccde', 
                 [('equal',   0,1, 0,1),
+                 ('replace', 1,5, 1,5),
+                 ('equal',   5,6, 5,6),
+                ])
+        chk_ops('aBcDec', 'abcdec', 
+                [('equal',   0,1, 0,1),
                  ('replace', 1,2, 1,2),
-                 ('equal',   2,4, 2,4),
-                 ('replace', 4,5, 4,5),
-                 ('equal',   5,6, 5,6)
+                 ('equal',   2,3, 2,3),
+                 ('replace', 3,4, 3,4),
+                 ('equal',   4,6, 4,6),
                 ])
         chk_ops('aBcdEcdFg', 'abcdecdfg', 
                 [('equal',   0,1, 0,1),
-                 ('replace', 1,2, 1,2),
-                 ('equal',   2,4, 2,4),
-                 ('replace', 4,5, 4,5),
-                 ('equal',   5,7, 5,7),
-                 ('replace', 7,8, 7,8),
+                 ('replace', 1,8, 1,8),
                  ('equal',   8,9, 8,9)
                 ])
+        chk_ops('aBcdEeXcdFg', 'abcdecdfg', 
+                [('equal',   0,1, 0,1),
+                 ('replace', 1,2, 1,2),
+                 ('equal',   2,4, 2,4),
+                 ('delete', 4,5, 4,4),
+                 ('equal',   5,6, 4,5),
+                 ('delete', 6,7, 5,5),
+                 ('equal',   7,9, 5,7),
+                 ('replace', 9,10, 7,8),
+                 ('equal',   10,11, 8,9)
+                ])
 
     def test_multiple_ranges(self):
         # There was an earlier bug where we used a bad set of ranges,
         # this triggers that specific bug, to make sure it doesn't regress
-        def chk_blocks(a, b, matching):
+        def chk_blocks(a, b, expected_blocks):
             # difflib always adds a signature of the total
             # length, with no matching entries at the end
-            s = SequenceMatcher(None, a, b)
+            s = bzrlib.patiencediff.PatienceSequenceMatcher(None, a, b)
             blocks = s.get_matching_blocks()
             x = blocks.pop()
             self.assertEquals(x, (len(a), len(b), 0))
-            self.assertEquals(matching, blocks)
+            self.assertEquals(expected_blocks, blocks)
 
         chk_blocks('abcdefghijklmnop'
                  , 'abcXghiYZQRSTUVWXYZijklmnop'
@@ -222,61 +259,63 @@
 
         chk_blocks('ABCd efghIjk  L'
                  , 'AxyzBCn mo pqrstuvwI1 2  L'
-                 , [(0,0,1), (1, 4, 2), (4, 7, 1), (9, 19, 1), (12, 23, 3)])
+                 , [(0,0,1), (1, 4, 2), (9, 19, 1), (12, 23, 3)])
 
+        # These are rot13 code snippets.
         chk_blocks('''\
-    get added when you add a file in the directory.
-    """
-    takes_args = ['file*']
-    takes_options = ['no-recurse']
-    
-    def run(self, file_list, no_recurse=False):
-        from bzrlib.add import smart_add, add_reporter_print, add_reporter_null
-        if is_quiet():
-            reporter = add_reporter_null
-        else:
-            reporter = add_reporter_print
-        smart_add(file_list, not no_recurse, reporter)
-
-
-class cmd_mkdir(Command):
-'''.splitlines(True)
-, '''\
-    get added when you add a file in the directory.
-
-    --dry-run will show which files would be added, but not actually 
-    add them.
-    """
-    takes_args = ['file*']
-    takes_options = ['no-recurse', 'dry-run']
-
-    def run(self, file_list, no_recurse=False, dry_run=False):
-        import bzrlib.add
-
-        if dry_run:
-            if is_quiet():
-                # This is pointless, but I'd rather not raise an error
-                action = bzrlib.add.add_action_null
-            else:
-                action = bzrlib.add.add_action_print
-        elif is_quiet():
-            action = bzrlib.add.add_action_add
-        else:
-            action = bzrlib.add.add_action_add_and_print
-
-        bzrlib.add.smart_add(file_list, not no_recurse, action)
-
-
-class cmd_mkdir(Command):
+    trg nqqrq jura lbh nqq n svyr va gur qverpgbel.
+    """
+    gnxrf_netf = ['svyr*']
+    gnxrf_bcgvbaf = ['ab-erphefr']
+  
+    qrs eha(frys, svyr_yvfg, ab_erphefr=Snyfr):
+        sebz omeyvo.nqq vzcbeg fzneg_nqq, nqq_ercbegre_cevag, nqq_ercbegre_ahyy
+        vs vf_dhvrg():
+            ercbegre = nqq_ercbegre_ahyy
+        ryfr:
+            ercbegre = nqq_ercbegre_cevag
+        fzneg_nqq(svyr_yvfg, abg ab_erphefr, ercbegre)
+
+
+pynff pzq_zxqve(Pbzznaq):
+'''.splitlines(True), '''\
+    trg nqqrq jura lbh nqq n svyr va gur qverpgbel.
+
+    --qel-eha jvyy fubj juvpu svyrf jbhyq or nqqrq, ohg abg npghnyyl 
+    nqq gurz.
+    """
+    gnxrf_netf = ['svyr*']
+    gnxrf_bcgvbaf = ['ab-erphefr', 'qel-eha']
+
+    qrs eha(frys, svyr_yvfg, ab_erphefr=Snyfr, qel_eha=Snyfr):
+        vzcbeg omeyvo.nqq
+
+        vs qel_eha:
+            vs vf_dhvrg():
+                # Guvf vf cbvagyrff, ohg V'q engure abg envfr na reebe
+                npgvba = omeyvo.nqq.nqq_npgvba_ahyy
+            ryfr:
+  npgvba = omeyvo.nqq.nqq_npgvba_cevag
+        ryvs vf_dhvrg():
+            npgvba = omeyvo.nqq.nqq_npgvba_nqq
+        ryfr:
+       npgvba = omeyvo.nqq.nqq_npgvba_nqq_naq_cevag
+
+        omeyvo.nqq.fzneg_nqq(svyr_yvfg, abg ab_erphefr, npgvba)
+
+
+pynff pzq_zxqve(Pbzznaq):
 '''.splitlines(True)
 , [(0,0,1), (1, 4, 2), (9, 19, 1), (12, 23, 3)])
 
-    def test_cdv_unified_diff(self):
+    def test_patience_unified_diff(self):
         txt_a = ['hello there\n',
                  'world\n',
                  'how are you today?\n']
         txt_b = ['hello there\n',
                  'how are you today?\n']
+        unified_diff = bzrlib.patiencediff.unified_diff
+        psm = bzrlib.patiencediff.PatienceSequenceMatcher
         self.assertEquals([ '---  \n',
                            '+++  \n',
                            '@@ -1,3 +1,2 @@\n',
@@ -284,8 +323,8 @@
                            '-world\n',
                            ' how are you today?\n'
                           ]
-                          , list(unified_diff(txt_a, txt_b
-                                        , sequencematcher=SequenceMatcher)))
+                          , list(unified_diff(txt_a, txt_b,
+                                 sequencematcher=psm)))
         txt_a = map(lambda x: x+'\n', 'abcdefghijklmnop')
         txt_b = map(lambda x: x+'\n', 'abcdefxydefghijklmnop')
         # This is the result with LongestCommonSubstring matching
@@ -304,7 +343,7 @@
                            ' e\n',
                            ' f\n']
                           , list(unified_diff(txt_a, txt_b)))
-        # And the cdv diff
+        # And the patience diff
         self.assertEquals(['---  \n',
                            '+++  \n',
                            '@@ -4,6 +4,11 @@\n',
@@ -321,12 +360,12 @@
                            ' i\n',
                           ]
                           , list(unified_diff(txt_a, txt_b,
-                                 sequencematcher=SequenceMatcher)))
-
-
-class TestCDVDiffLibFiles(TestCaseInTempDir):
-
-    def test_cdv_unified_diff_files(self):
+                                 sequencematcher=psm)))
+
+
+class TestPatienceDiffLibFiles(TestCaseInTempDir):
+
+    def test_patience_unified_diff_files(self):
         txt_a = ['hello there\n',
                  'world\n',
                  'how are you today?\n']
@@ -335,6 +374,8 @@
         open('a1', 'wb').writelines(txt_a)
         open('b1', 'wb').writelines(txt_b)
 
+        unified_diff_files = bzrlib.patiencediff.unified_diff_files
+        psm = bzrlib.patiencediff.PatienceSequenceMatcher
         self.assertEquals(['--- a1 \n',
                            '+++ b1 \n',
                            '@@ -1,3 +1,2 @@\n',
@@ -343,7 +384,7 @@
                            ' how are you today?\n',
                           ]
                           , list(unified_diff_files('a1', 'b1',
-                                 sequencematcher=SequenceMatcher)))
+                                 sequencematcher=psm)))
 
         txt_a = map(lambda x: x+'\n', 'abcdefghijklmnop')
         txt_b = map(lambda x: x+'\n', 'abcdefxydefghijklmnop')
@@ -367,7 +408,7 @@
                            ' f\n']
                           , list(unified_diff_files('a2', 'b2')))
 
-        # And the cdv diff
+        # And the patience diff
         self.assertEquals(['--- a2 \n',
                            '+++ b2 \n',
                            '@@ -4,6 +4,11 @@\n',
@@ -384,4 +425,4 @@
                            ' i\n',
                           ]
                           , list(unified_diff_files('a2', 'b2',
-                                 sequencematcher=SequenceMatcher)))
+                                 sequencematcher=psm)))

=== modified file 'bzrlib/tests/test_merge3.py'
--- bzrlib/tests/test_merge3.py	2006-05-20 17:51:13 +0000
+++ bzrlib/tests/test_merge3.py	2006-05-29 01:17:43 +0000
@@ -302,7 +302,7 @@
         self.log(''.join(ml))
         self.assertEquals(ml, MERGED_RESULT)
 
-    def test_minimal_conflicts(self):
+    def test_minimal_conflicts_common(self):
         """Reprocessing"""
         base_text = ("a\n" * 20).splitlines(True)
         this_text = ("a\n"*10+"b\n" * 10).splitlines(True)
@@ -310,10 +310,46 @@
         m3 = Merge3(base_text, other_text, this_text)
         m_lines = m3.merge_lines('OTHER', 'THIS', reprocess=True)
         merged_text = "".join(list(m_lines))
-        optimal_text = "a\n" * 10 + "<<<<<<< OTHER\nc\n=======\n>>>>>>> THIS"\
-            + "\n" + 8* "b\n" + "<<<<<<< OTHER\nc\n=======\nb\nb\n>>>>>>>"\
-            + " THIS\n"
-        self.assertEqualDiff(merged_text, optimal_text)
+        optimal_text = ("a\n" * 10 + "<<<<<<< OTHER\nc\n"
+            + 8* "b\n" + "c\n=======\n"
+            + 10*"b\n" + ">>>>>>> THIS\n")
+        self.assertEqualDiff(optimal_text, merged_text)
+
+    def test_minimal_conflicts_unique(self):
+        def add_newline(s):
+            """Add a newline to each entry in the string"""
+            return [(x+'\n') for x in s]
+
+        base_text = add_newline("abcdefghijklm")
+        this_text = add_newline("abcdefghijklmNOPQRSTUVWXYZ")
+        other_text = add_newline("abcdefghijklm1OPQRSTUVWXY2")
+        m3 = Merge3(base_text, other_text, this_text)
+        m_lines = m3.merge_lines('OTHER', 'THIS', reprocess=True)
+        merged_text = "".join(list(m_lines))
+        optimal_text = ''.join(add_newline("abcdefghijklm")
+            + ["<<<<<<< OTHER\n1\n=======\nN\n>>>>>>> THIS\n"]
+            + add_newline('OPQRSTUVWXY')
+            + ["<<<<<<< OTHER\n2\n=======\nZ\n>>>>>>> THIS\n"]
+            )
+        self.assertEqualDiff(optimal_text, merged_text)
+
+    def test_minimal_conflicts_nonunique(self):
+        def add_newline(s):
+            """Add a newline to each entry in the string"""
+            return [(x+'\n') for x in s]
+
+        base_text = add_newline("abacddefgghij")
+        this_text = add_newline("abacddefgghijkalmontfprz")
+        other_text = add_newline("abacddefgghijknlmontfprd")
+        m3 = Merge3(base_text, other_text, this_text)
+        m_lines = m3.merge_lines('OTHER', 'THIS', reprocess=True)
+        merged_text = "".join(list(m_lines))
+        optimal_text = ''.join(add_newline("abacddefgghijk")
+            + ["<<<<<<< OTHER\nn\n=======\na\n>>>>>>> THIS\n"]
+            + add_newline('lmontfpr')
+            + ["<<<<<<< OTHER\nd\n=======\nz\n>>>>>>> THIS\n"]
+            )
+        self.assertEqualDiff(optimal_text, merged_text)
 
     def test_reprocess_and_base(self):
         """Reprocessing and showing base breaks correctly"""

=== modified file 'bzrlib/textmerge.py'
--- bzrlib/textmerge.py	2006-05-25 22:01:44 +0000
+++ bzrlib/textmerge.py	2006-05-29 01:00:30 +0000
@@ -18,7 +18,7 @@
 #         Aaron Bentley <aaron.bentley@utoronto.ca>
 
 
-from bzrlib.patiencediff import SequenceMatcher
+import bzrlib.patiencediff
 
 
 class TextMerge(object):
@@ -131,7 +131,7 @@
         """Return structured merge info.  
         See TextMerge docstring.
         """
-        sm = SequenceMatcher(None, self.lines_a, self.lines_b)
+        sm = bzrlib.patiencediff.PatienceSequenceMatcher(None, self.lines_a, self.lines_b)
         pos_a = 0
         pos_b = 0
         for ai, bi, l in sm.get_matching_blocks():

=== modified file 'bzrlib/weave.py'
--- bzrlib/weave.py	2006-05-23 14:42:44 +0000
+++ bzrlib/weave.py	2006-05-29 01:00:30 +0000
@@ -70,7 +70,6 @@
 
 from copy import copy
 from cStringIO import StringIO
-from difflib import SequenceMatcher
 import os
 import sha
 import time
@@ -84,7 +83,7 @@
         )
 import bzrlib.errors as errors
 from bzrlib.osutils import sha_strings
-from bzrlib.patiencediff import SequenceMatcher, unified_diff
+import bzrlib.patiencediff
 from bzrlib.symbol_versioning import *
 from bzrlib.tsort import topo_sort
 from bzrlib.versionedfile import VersionedFile, InterVersionedFile
@@ -192,7 +191,7 @@
         self._name_map = {}
         self._weave_name = weave_name
         if matcher is None:
-            self._matcher = SequenceMatcher
+            self._matcher = bzrlib.patiencediff.PatienceSequenceMatcher
         else:
             self._matcher = matcher
 
@@ -1366,7 +1365,7 @@
         v1, v2 = map(int, argv[3:5])
         lines1 = w.get(v1)
         lines2 = w.get(v2)
-        diff_gen = unified_diff(lines1, lines2,
+        diff_gen = bzrlib.patiencediff.unified_diff(lines1, lines2,
                                 '%s version %d' % (fn, v1),
                                 '%s version %d' % (fn, v2))
         sys.stdout.writelines(diff_gen)

=== modified file 'patience-test.py'
--- patience-test.py	2006-05-23 02:13:13 +0000
+++ patience-test.py	2006-05-28 22:00:33 +0000
@@ -21,7 +21,7 @@
 from tempfile import mkdtemp
 
 from bzrlib.branch import Branch
-from bzrlib.patiencediff import SequenceMatcher
+import bzrlib.patiencediff
 from bzrlib.diff import internal_diff
 from bzrlib.osutils import pathjoin
 
@@ -62,7 +62,7 @@
                 new_patch = StringIO()
                 try:
                     internal_diff('old', old_lines, 'new', new_lines, new_patch,
-                                  sequence_matcher=SequenceMatcher)
+                        sequence_matcher=bzrlib.patiencediff.PatienceSequenceMatcher)
                 except:
                     file(pathjoin(temp_dir, 'old'), 
                          'wb').write(''.join(old_lines))

